'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.createInput = createInput;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _InputBasic = require('./InputBasic');

var _InputBasic2 = _interopRequireDefault(_InputBasic);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function createInput(defaultMetadata) {
	function PhoneInput(_ref, ref) {
		var country = _ref.country,
		    value = _ref.value,
		    onChange = _ref.onChange,
		    rest = _objectWithoutProperties(_ref, ['country', 'value', 'onChange']);

		var onChangeHandler = (0, _react.useCallback)(function (value) {
			if (country) {
				// Force an absence of `+` in the beginning of a `value`
				// when a `country` has been specified.
				if (value && value[0] === '+') {
					value = value.slice(1);
				}
			} else {
				// Force a `+` in the beginning of a `value`
				// when no `country` has been specified.
				if (value && value[0] !== '+') {
					value = '+' + value;
				}
			}
			if (value === '') {
				value = undefined;
			}
			onChange(value);
		}, [onChange, country]);
		return _react2.default.createElement(_InputBasic2.default, _extends({}, rest, {
			ref: ref,
			country: country,
			value: value || '',
			onChange: onChangeHandler }));
	}

	PhoneInput = _react2.default.forwardRef(PhoneInput);

	PhoneInput.propTypes = {
		/**
   * HTML `<input/>` `type` attribute.
   */
		type: _propTypes2.default.string,

		/**
   * HTML `<input/>` `autocomplete` attribute.
   */
		autoComplete: _propTypes2.default.string,

		/**
   * A two-letter country code for formatting `value`
   * as a national phone number (e.g. `(800) 555 35 35`).
   * Examples: "US", "RU", etc.
   * If no `country` is passed then `value`
   * is formatted as an international phone number.
   * (for example, `+7 800 555 35 35`)
   */
		country: _propTypes2.default.string,

		/**
   * The parsed phone number.
   * Examples: `undefined`, `""`, `"+"`, `"+123"`, `"123"`.
   */
		value: _propTypes2.default.string,

		/**
   * Updates the `value`.
   */
		onChange: _propTypes2.default.func.isRequired,

		/**
   * The `<input/>` component.
   */
		inputComponent: _propTypes2.default.elementType,

		/**
   * `libphonenumber-js` metadata.
   */
		metadata: _propTypes2.default.object.isRequired
	};

	PhoneInput.defaultProps = {
		/**
   * HTML `<input/>` `type="tel"`.
   */
		type: 'tel',

		/**
   * Remember (and autofill) the value as a phone number.
   */
		autoComplete: 'tel',

		/**
   * `libphonenumber-js` metadata.
   */
		metadata: defaultMetadata
	};

	return PhoneInput;
}

exports.default = createInput();
//# sourceMappingURL=PhoneInputNoCountrySelect.js.map